@@ -5,7 +5,7 @@ from trytond.backend.database import DatabaseInterface, CursorInterface
 from trytond.config import CONFIG
 from trytond.session import Session
 from psycopg2.pool import ThreadedConnectionPool
-from psycopg2.extensions import cursor
+from psycopg2.extensions import cursor as PsycopgCursor
 from psycopg2.extensions import ISOLATION_LEVEL_SERIALIZABLE
 from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
 from psycopg2.extensions import register_type, register_adapter
@@ -18,6 +18,7 @@ import re
 import os
 if os.name == 'posix':
     import pwd
+from decimal import Decimal
 
 RE_FROM = re.compile('.* from "?([a-zA-Z_0-9]+)"?.*$')
 RE_INTO = re.compile('.* into "?([a-zA-Z_0-9]+)"?.*$')
@@ -124,7 +125,7 @@ class Database(DatabaseInterface):
 
         if os.name == "nt":
             tmpfile = (os.environ['TMP'] or 'C:\\') + os.tmpnam()
-            file(tmpfile, 'wb').write(data)
+            open(tmpfile, 'wb').write(data)
             args2 = list(args2)
             args2.append(' ' + tmpfile)
             args2 = tuple(args2)
@@ -207,7 +208,7 @@ class Database(DatabaseInterface):
     def init(cursor):
         from trytond.tools import safe_eval
         sql_file = os.path.join(os.path.dirname(__file__), 'init.sql')
-        for line in file(sql_file).read().split(';'):
+        for line in open(sql_file).read().split(';'):
             if (len(line)>0) and (not line.isspace()):
                 cursor.execute(line)
 
@@ -215,7 +216,7 @@ class Database(DatabaseInterface):
             root_path = os.path.join(os.path.dirname(__file__), '..', '..')
             tryton_file = os.path.join(root_path, i, '__tryton__.py')
             mod_path = os.path.join(root_path, i)
-            info = safe_eval(file(tryton_file).read())
+            info = safe_eval(open(tryton_file).read())
             active = info.get('active', False)
             if active:
                 state = 'to install'
@@ -238,13 +239,11 @@ class Database(DatabaseInterface):
                         (0, module_id, dependency))
 
 
-class _Cursor(cursor):
+class _Cursor(PsycopgCursor):
 
     def __build_dict(self, row):
-        res = {}
-        for i in range(len(self.description)):
-            res[self.description[i][0]] = row[i]
-        return res
+        return dict((desc[0], row[i])
+                for i, desc in enumerate(self.description))
 
     def dictfetchone(self):
         row = self.fetchone()
@@ -254,18 +253,12 @@ class _Cursor(cursor):
             return row
 
     def dictfetchmany(self, size):
-        res = []
         rows = self.fetchmany(size)
-        for row in rows:
-            res.append(self.__build_dict(row))
-        return res
+        return [self.__build_dict(row) for row in rows]
 
     def dictfetchall(self):
-        res = []
         rows = self.fetchall()
-        for row in rows:
-            res.append(self.__build_dict(row))
-        return res
+        return [self.__build_dict(row) for row in rows]
 
 
 class Cursor(CursorInterface):
@@ -388,8 +381,8 @@ class Cursor(CursorInterface):
         self.cursor.execute('SELECT last_value FROM "' + table + '_id_seq"')
         return self.cursor.fetchone()[0]
 
-    def has_lock(self):
-        return True
+    def lock(self, table):
+        self.cursor.execute('LOCK "%s"' % table)
 
     def has_constraint(self):
         return True
@@ -397,9 +390,11 @@ class Cursor(CursorInterface):
     def limit_clause(self, select, limit=None, offset=None):
         if limit is not None:
             select += ' LIMIT %d' % limit
-        if offset is not None:
+        if offset is not None and offset != 0:
             select += ' OFFSET %d' % offset
         return select
 
 register_type(UNICODE)
 register_adapter(Session, AsIs)
+register_adapter(float, lambda value: AsIs(repr(value)))
+register_adapter(Decimal, lambda value: AsIs(str(value)))
