@@ -2,7 +2,6 @@
 #this repository contains the full copyright notices and license terms.
 
 from trytond.backend.table import TableHandlerInterface
-from trytond.backend.postgresql.fields import FIELDS
 import logging
 
 
@@ -14,7 +13,7 @@ class TableHandler(TableHandlerInterface):
         self._columns = {}
         self._constraints = []
         self._fk_deltypes = {}
-        self._indexes = {}
+        self._indexes = []
         self._field2module = {}
 
         # Create sequence if necessary
@@ -25,6 +24,9 @@ class TableHandler(TableHandlerInterface):
         if not self.table_exist(self.cursor, self.table_name):
             self.cursor.execute('CREATE TABLE "%s" ()' % self.table_name)
 
+        self.cursor.execute('COMMENT ON TABLE "%s" IS \'%s\'' % 
+                (self.table_name, model._description.replace("'", "''")))
+
         self._update_definitions()
         if 'id' not in self._columns:
             if not self.history:
@@ -133,13 +135,14 @@ class TableHandler(TableHandlerInterface):
                 "size": size,
                 "typname": typname}
 
-        # fetch constrains for the table
+        # fetch constraints for the table
         self.cursor.execute("SELECT co.contype, co.confdeltype, at.attname, "\
                          "cl2.relname, co.conname "\
                        "FROM pg_constraint co "\
-                         "LEFT JOIN pg_class cl on (co.conrelid = cl.oid) "\
-                         "LEFT JOIN pg_class cl2 on (co.confrelid = cl2.oid) "\
-                         "LEFT JOIN pg_attribute at on (co.conkey[1] = at.attnum) "\
+                         "LEFT JOIN pg_class cl ON (co.conrelid = cl.oid) "\
+                         "LEFT JOIN pg_class cl2 ON (co.confrelid = cl2.oid) "\
+                         "LEFT JOIN pg_attribute at ON " \
+                            "(co.conkey[1] = at.attnum) "\
                        "WHERE cl.relname = %s AND at.attrelid = cl.oid",
                        (self.table_name,))
         self._constraints = []
@@ -176,16 +179,16 @@ class TableHandler(TableHandlerInterface):
 
         self.cursor.execute("ALTER TABLE \"%s\" " \
                        "RENAME COLUMN \"%s\" " \
-                       "TO temp_change_size" % \
+                       "TO _temp_change_size" % \
                        (self.table_name, column_name))
         self.cursor.execute("ALTER TABLE \"%s\" " \
                        "ADD COLUMN \"%s\" %s" % \
                        (self.table_name, column_name, column_type))
         self.cursor.execute("UPDATE \"%s\" " \
-                       "SET \"%s\" = temp_change_size::%s" % \
+                       "SET \"%s\" = _temp_change_size::%s" % \
                        (self.table_name, column_name, column_type))
         self.cursor.execute("ALTER TABLE \"%s\" " \
-                       "DROP COLUMN temp_change_size" % \
+                       "DROP COLUMN _temp_change_size" % \
                        (self.table_name,))
         self._update_definitions()
 
@@ -200,8 +203,16 @@ class TableHandler(TableHandlerInterface):
                 (value,))
 
     def add_raw_column(self, column_name, column_type, column_format,
-            default_fun=None, field_size=None, migrate=True):
+            default_fun=None, field_size=None, migrate=True, string=''):
+        def comment():
+            self.cursor.execute('COMMENT ON COLUMN "%s"."%s" IS \'%s\'' %
+                    (self.table_name, column_name, string.replace("'", "''")))
         if self.column_exist(column_name):
+            if column_name in ('create_date', 'write_date'):
+                #Migrate dates from timestamp(0) to timestamp
+                self.cursor.execute('ALTER TABLE "' + self.table_name + '" ' \
+                        'ALTER COLUMN "' + column_name + '" TYPE timestamp')
+            comment()
             if not migrate:
                 return
             base_type = column_type[0].lower()
@@ -226,7 +237,6 @@ class TableHandler(TableHandlerInterface):
                 if field_size == None:
                     if self._columns[column_name]['size'] > 0:
                         self.alter_size(column_name, base_type)
-                    pass
                 elif self._columns[column_name]['size'] == field_size:
                     pass
                 elif self._columns[column_name]['size'] > 0 and \
@@ -245,6 +255,7 @@ class TableHandler(TableHandlerInterface):
         column_type = column_type[1]
         self.cursor.execute('ALTER TABLE "%s" ADD COLUMN "%s" %s' %
                        (self.table_name, column_name, column_type))
+        comment()
 
         if column_format:
             # check if table is non-empty:
@@ -306,7 +317,7 @@ class TableHandler(TableHandlerInterface):
                 return
             self.cursor.execute('CREATE INDEX "' + index_name + '" ' \
                                'ON "' + self.table_name + '" ( ' + \
-                               ','.join(['"' + x + '"' for x in column_name]) + \
+                               ','.join(['"' + x + '"' for x in column_name]) +\
                                ')')
             self._update_definitions()
         elif action == 'remove':
@@ -342,9 +353,11 @@ class TableHandler(TableHandlerInterface):
                         'of table %s not null !\n'\
                         'Try to re-run: ' \
                         'trytond.py --update=module\n' \
-                        'If it doesn\'t work, update records and execute manually:\n' \
-                        'ALTER TABLE "%s" ALTER COLUMN "%s" SET NOT NULL' % \
-                        (column_name, self.table_name, self.table_name, column_name))
+                        'If it doesn\'t work, update records ' \
+                        'and execute manually:\n' \
+                        'ALTER TABLE "%s" ALTER COLUMN "%s" SET NOT NULL' %
+                        (column_name, self.table_name, self.table_name,
+                            column_name))
         elif action == 'remove':
             if not self._columns[column_name]['notnull']:
                 return
@@ -412,7 +425,7 @@ class TableHandler(TableHandlerInterface):
         self._update_definitions()
 
     @staticmethod
-    def dropTable(cursor, model, table, cascade=False):
+    def drop_table(cursor, model, table, cascade=False):
         cursor.execute('DELETE from ir_model_data where '\
                            'model = \'%s\'' % model)
 
