@@ -4,12 +4,11 @@
 from trytond.backend.database import DatabaseInterface, CursorInterface
 from trytond.config import CONFIG
 from trytond.session import Session
-import logging
 import os
 import re
-import mx.DateTime
 from decimal import Decimal
 import datetime
+import time
 
 _FIX_ROWCOUNT = False
 try:
@@ -35,49 +34,53 @@ def extract(lookup_type, date):
         hours, minutes, seconds = map(int, timepart_full[0].split(":"))
         if len(timepart_full) == 2:
             microseconds = int(timepart_full[1])
-            seconds += microseconds / 1000000.0
-        date = mx.DateTime.DateTime(year, month, day, hours, minutes, seconds)
+        date = datetime.datetime(year, month, day, hours, minutes, seconds,
+                microseconds)
     except:
         return None
     if lookup_type.lower() == 'century':
         return date.year / 100 + (date.year % 100 and 1 or 0)
     elif lookup_type.lower() == 'decade':
-        return date.year.year / 10
+        return date.year / 10
     elif lookup_type.lower() == 'dow':
         return (date.weekday() + 1) % 7
     elif lookup_type.lower() == 'doy':
-        return date.day_of_year
+        return date.timetuple().tm_yday
     elif lookup_type.lower() == 'epoch':
-        return date.ticks()
+        return int(time.mktime(date.timetuple()))
     elif lookup_type.lower() == 'microseconds':
-        return int(a.second * 1000000)
+        return date.microsecond
     elif lookup_type.lower() == 'millennium':
         return date.year / 1000 + (date.year % 1000 and 1 or 0)
     elif lookup_type.lower() == 'milliseconds':
-        return int(a.second * 1000)
+        return date.microsecond / 1000
     elif lookup_type.lower() == 'quarter':
         return date.month / 4 + 1
     elif lookup_type.lower() == 'week':
-        return date.iso_week[1]
+        return date.isocalendar()[1]
     return getattr(date, lookup_type)
 
 def date_trunc(_type, date):
     if _type == 'second':
         return date
     try:
-        date = mx.DateTime.strptime(date, '%Y-%m-%d %H:%M:%S')
+        tm_tuple = time.strptime(date, '%Y-%m-%d %H:%M:%S')
     except:
         return None
     if _type == 'year':
-        return "%i-01-01 00:00:00" % dt.year
+        return "%i-01-01 00:00:00" % tm_tuple.tm_year
     elif _type == 'month':
-        return "%i-%02i-01 00:00:00" % (dt.year, dt.month)
+        return "%i-%02i-01 00:00:00" % (tm_tuple.tm_year, tm_tuple.tm_mon)
     elif _type == 'day':
-        return "%i-%02i-%02i 00:00:00" % (dt.year, dt.month, dt.day)
+        return "%i-%02i-%02i 00:00:00" % (tm_tuple.tm_year, tm_tuple.tm_mon,
+                tm_tuple.tm_mday)
 
 def split_part(text, delimiter, count):
     return (text.split(delimiter) + [''] * (count - 1))[count - 1]
 
+def replace(text, pattern, replacement):
+    return str(text).replace(pattern, replacement)
+
 
 class Database(DatabaseInterface):
 
@@ -109,6 +112,8 @@ class Database(DatabaseInterface):
         self._conn.create_function('extract', 2, extract)
         self._conn.create_function('date_trunc', 2, date_trunc)
         self._conn.create_function('split_part', 3, split_part)
+        if sqlite.sqlite_version_info < (3, 3, 14):
+            self._conn.create_function('replace', 3, replace)
         return self
 
     def cursor(self, autocommit=False):
@@ -157,7 +162,7 @@ class Database(DatabaseInterface):
             raise Exception('Wrong database name!')
         path = os.path.join(CONFIG['data_path'],
                 database_name + '.sqlite')
-        file_p = file(path, 'rb')
+        file_p = open(path, 'rb')
         data = file_p.read()
         file_p.close()
         return data
@@ -172,19 +177,24 @@ class Database(DatabaseInterface):
                 database_name + '.sqlite')
         if os.path.isfile(path):
             raise Exception('Database already exists!')
-        file_p = file(path, 'wb')
+        file_p = open(path, 'wb')
         file_p.write(data)
         file_p.close()
 
     @staticmethod
     def list(cursor):
         res = []
-        for file in os.listdir(CONFIG['data_path']) + [':memory:']:
-            if file.endswith('.sqlite') or file == ':memory:':
-                if file == ':memory:':
+        listdir = [':memory:']
+        try:
+            listdir += os.listdir(CONFIG['data_path'])
+        except OSError:
+            pass
+        for db_file in listdir:
+            if db_file.endswith('.sqlite') or db_file == ':memory:':
+                if db_file == ':memory:':
                     db_name = ':memory:'
                 else:
-                    db_name = file[:-7]
+                    db_name = db_file[:-7]
                 try:
                     database = Database(db_name)
                 except:
@@ -199,7 +209,7 @@ class Database(DatabaseInterface):
     def init(cursor):
         from trytond.tools import safe_eval
         sql_file = os.path.join(os.path.dirname(__file__), 'init.sql')
-        for line in file(sql_file).read().split(';'):
+        for line in open(sql_file).read().split(';'):
             if (len(line)>0) and (not line.isspace()):
                 cursor.execute(line)
 
@@ -207,7 +217,7 @@ class Database(DatabaseInterface):
             root_path = os.path.join(os.path.dirname(__file__), '..', '..')
             tryton_file = os.path.join(root_path, i, '__tryton__.py')
             mod_path = os.path.join(root_path, i)
-            info = safe_eval(file(tryton_file).read())
+            info = safe_eval(open(tryton_file).read())
             active = info.get('active', False)
             if active:
                 state = 'to install'
@@ -233,10 +243,8 @@ class Database(DatabaseInterface):
 class _Cursor(sqlite.Cursor):
 
     def __build_dict(self, row):
-        res = {}
-        for i in range(len(self.description)):
-            res[self.description[i][0].strip('"')] = row[i]
-        return res
+        return dict((desc[0], row[i])
+                for i, desc in enumerate(self.description))
 
     def dictfetchone(self):
         row = self.fetchone()
@@ -246,18 +254,12 @@ class _Cursor(sqlite.Cursor):
             return row
 
     def dictfetchmany(self, size):
-        res = []
         rows = self.fetchmany(size)
-        for row in rows:
-            res.append(self.__build_dict(row))
-        return res
+        return [self.__build_dict(row) for row in rows]
 
     def dictfetchall(self):
-        res = []
         rows = self.fetchall()
-        for row in rows:
-            res.append(self.__build_dict(row))
-        return res
+        return [self.__build_dict(row) for row in rows]
 
 
 class Cursor(CursorInterface):
@@ -277,12 +279,12 @@ class Cursor(CursorInterface):
 
     def execute(self, sql, params=None):
         buf = ""
-        for nq, q in QUOTE_SEPARATION.findall(sql+"''"):
-            nq = nq.replace('?', '??')
-            nq = nq.replace('%s', '?')
-            nq = nq.replace('ilike', 'like')
-            nq = re.sub(EXTRACT_PATTERN, r'EXTRACT("\1",', nq)
-            buf += nq + q
+        for nquote, quote in QUOTE_SEPARATION.findall(sql+"''"):
+            nquote = nquote.replace('?', '??')
+            nquote = nquote.replace('%s', '?')
+            nquote = nquote.replace('ilike', 'like')
+            nquote = re.sub(EXTRACT_PATTERN, r'EXTRACT("\1",', nquote)
+            buf += nquote + quote
         sql = buf[:-2]
         try:
             if params:
@@ -336,8 +338,8 @@ class Cursor(CursorInterface):
         self.cursor.execute('SELECT last_insert_rowid()')
         return self.cursor.fetchone()[0]
 
-    def has_lock(self):
-        return False
+    def lock(self, table):
+        pass
 
     def has_constraint(self):
         return False
@@ -351,9 +353,9 @@ class Cursor(CursorInterface):
             select += ' OFFSET %d' % offset
         return select
 
-sqlite.register_converter('NUMERIC', lambda val: Decimal(str(val)))
-sqlite.register_adapter(Decimal, lambda val: float(val))
-sqlite.register_adapter(Session, lambda val: int(val))
+sqlite.register_converter('NUMERIC', lambda val: Decimal(val))
+sqlite.register_adapter(Decimal, lambda val: buffer(str(val)))
+sqlite.register_adapter(Session, int)
 def adapt_datetime(val):
     return val.replace(tzinfo=None).isoformat(" ")
 sqlite.register_adapter(datetime.datetime, adapt_datetime)
