@@ -3,22 +3,13 @@
 
 from trytond.model import fields
 from trytond.pool import Pool
+from trytond.pyson import PYSONEncoder
 import copy
 
 
 class Model(object):
     """
     Define a model in Tryton.
-
-    :_name: a unique string to reference the model throughout the platform.
-    :_inherits: a model from which this one inherits.
-    :_description: a non technical description string.
-    :_rpc: a dictionary with a method name allowed to be called remotely and
-        a boolean as value that defines if it must be called in a commited
-        transaction.
-    :_error_messages: a dictionary mapping keywords to an error message. Eg.:
-
-        ``{'recursive_categories': 'You can not create recursive categories!'}``
     """
     _name = None
     _inherits = {}
@@ -27,6 +18,9 @@ class Model(object):
     __columns = None
     __xxx2many_targets = None
     __defaults = None
+    _rec_name = 'name'
+
+    id = fields.Integer('ID', readonly=True)
 
     def _reset_columns(self):
         self.__columns = None
@@ -37,7 +31,7 @@ class Model(object):
             return self.__columns
         res = {}
         for attr in dir(self):
-            if attr in ('_columns', '_defaults', '_xxx2many_targets'):
+            if attr.startswith('_'):
                 continue
             if isinstance(getattr(self, attr), fields.Field):
                 res[attr] = getattr(self, attr)
@@ -106,6 +100,35 @@ class Model(object):
                 if on_change_with not in self._rpc:
                     self._rpc[on_change_with] = False
 
+    def __getattr__(self, name):
+        try:
+            super(Model, self).__getattr__(name)
+        except AttributeError:
+            # Search if a function exists in inherits parents
+            for model_name, field_name in self._inherits.iteritems():
+                model_obj = self.pool.get(model_name)
+                if hasattr(model_obj, name) and \
+                        callable(getattr(model_obj, name)):
+                    def func(cursor, user, *args, **kwargs):
+                        if args:
+                            ids = args[0]
+                            int_id = False
+                            if isinstance(ids, (int, long)):
+                                int_id = True
+                                ids = [ids]
+                            if reduce(lambda x, y: x and
+                                    isinstance(y, (int, long)), ids, True):
+                                # Replace ids by the parent ids
+                                ids = [getattr(x, field_name).id for x in
+                                        self.browse(cursor, user, ids)]
+                            if int_id:
+                                ids = ids[0]
+                            args = tuple([ids] + list(args[1:]))
+                        return getattr(model_obj, name)(cursor, user, *args,
+                                **kwargs)
+                    return func
+            raise
+
     def _inherits_reload(self):
         """
         Reconstruct _inherit_fields
@@ -244,9 +267,9 @@ class Model(object):
                 'WHERE f.model = m.id ' \
                     'AND m.model = %s ',
                         (self._name,))
-        fields = {}
+        model_fields = {}
         for field in cursor.dictfetchall():
-            fields[field['name']] = field
+            model_fields[field['name']] = field
 
         # Prefetch field translations
         if self._columns:
@@ -277,17 +300,17 @@ class Model(object):
                 relation = field.model_name
             elif hasattr(field, 'relation_name'):
                 relation = field.relation_name
-            if field_name not in fields:
+            if field_name not in model_fields:
                 cursor.execute("INSERT INTO ir_model_field " \
                         "(model, name, field_description, ttype, " \
                             "relation, help, module) " \
                         "VALUES (%s, %s, %s, %s, %s, %s, %s)",
                         (model_id, field_name, field.string, field._type,
                             relation, field.help, module_name))
-            elif fields[field_name]['field_description'] != field.string \
-                    or fields[field_name]['ttype'] != field._type \
-                    or fields[field_name]['relation'] != relation \
-                    or fields[field_name]['help'] != field.help:
+            elif model_fields[field_name]['field_description'] != field.string \
+                    or model_fields[field_name]['ttype'] != field._type \
+                    or model_fields[field_name]['relation'] != relation \
+                    or model_fields[field_name]['help'] != field.help:
                 cursor.execute('UPDATE ir_model_field ' \
                         'SET field_description = %s, ' \
                             'ttype = %s, ' \
@@ -295,7 +318,7 @@ class Model(object):
                             'help = %s ' \
                         'WHERE id = %s ',
                         (field.string, field._type, relation,
-                            field.help, fields[field_name]['id']))
+                            field.help, model_fields[field_name]['id']))
             trans_name = self._name + ',' + field_name
             if trans_name not in trans_fields:
                 if field_name not in ('create_uid', 'create_date',
@@ -327,7 +350,7 @@ class Model(object):
                     and ((hasattr(field, 'translate_selection') \
                         and field.translate_selection)
                         or not hasattr(field, 'translate_selection')):
-                for (key, val) in field.selection:
+                for (_, val) in field.selection:
                     if trans_name not in trans_selection \
                             or val not in trans_selection[trans_name]:
                         cursor.execute('INSERT INTO ir_translation ' \
@@ -337,13 +360,14 @@ class Model(object):
                                 (trans_name, 'en_US', 'selection', val, '',
                                     module_name, False))
         # Clean ir_model_field from field that are no more existing.
-        for field_name in fields:
-            if fields[field_name]['module'] == module_name \
+        for field_name in model_fields:
+            if model_fields[field_name]['module'] == module_name \
                     and field_name not in self._columns:
-                # XXX This delete field even when it is defined later in the module
+                #XXX This delete field even when it is defined later
+                # in the module
                 cursor.execute('DELETE FROM ir_model_field '\
                                    'WHERE id = %s',
-                               (fields[field_name]['id'],))
+                               (model_fields[field_name]['id'],))
 
         # Add error messages in ir_translation
         cursor.execute('SELECT id, src FROM ir_translation ' \
@@ -564,8 +588,9 @@ class Model(object):
                     field_value2 = []
                     for i in range(len(field_value or [])):
                         field_value2.append({})
-                        for field2 in field_value[i]:
-                            if obj._columns[field2]._type \
+                        for field2 in field_value[i].keys():
+                            if field2 in obj._columns \
+                                    and obj._columns[field2]._type \
                                     in ('many2one',):
                                 obj2 = self.pool.get(
                                         obj._columns[field2].model_name)
@@ -613,8 +638,8 @@ class Model(object):
                                 and self._columns[arg]._type == 'many2one':
                             if isinstance(args[arg], (list, tuple)):
                                 args[arg] = args[arg][0]
-                    val.update(getattr(self, 'on_change_' + field)(cursor, user,
-                        [], args, context=context))
+                    val.update(getattr(self, 'on_change_' + field)(cursor,
+                        user, args, context=context))
                 if self._columns[field]._type in ('one2many',):
                     obj = self.pool.get(self._columns[field].model_name)
                     for val2 in res[field]:
@@ -625,7 +650,7 @@ class Model(object):
 
     def fields_get(self, cursor, user, fields_names=None, context=None):
         """
-        Return the definition of each field in the object.
+        Return the definition of each field on the model.
 
         :param cursor: the database cursor
         :param user: the user id
@@ -666,6 +691,8 @@ class Model(object):
                             val))
         translation_obj._get_sources(cursor, trans_args)
 
+        encoder = PYSONEncoder()
+
         for field in (x for x in self._columns.keys()
                 if ((not fields_names) or x in fields_names)):
             res[field] = {'type': self._columns[field]._type}
@@ -756,6 +783,11 @@ class Model(object):
                 res[field]['relation_field'] = copy.copy(
                         self._columns[field].field)
 
+            # convert attributes into pyson
+            for attr in ('states', 'domain', 'context', 'digits', 'add_remove'):
+                if attr in res[field]:
+                    res[field][attr] = encoder.encode(res[field][attr])
+
         if fields_names:
             # filter out fields which aren't in the fields_names list
             for i in res.keys():
