@@ -28,7 +28,6 @@ import traceback
 from trytond.config import CONFIG
 from trytond.backend import DatabaseIntegrityError
 import inspect
-import mx.DateTime
 import logging
 
 PARENTS = {
@@ -50,18 +49,30 @@ class ReportFactory:
 
 class TranslateFactory:
 
-    def __init__(self, cursor, report_name, language, translation):
+    def __init__(self, cursor, user, report_name, language, translation):
         self.cursor = cursor
+        self.user = user
         self.report_name = report_name
         self.language = language
         self.translation = translation
+        self.cache = {}
 
     def __call__(self, text):
-        res = self.translation._get_source(self.cursor,
-                self.report_name, 'odt', self.language, text)
-        if res:
-            return res
-        return text
+        if self.language not in self.cache:
+            self.cache[self.language] = {}
+            translation_ids = self.translation.search(self.cursor, self.user, [
+                ('lang', '=', self.language),
+                ('type', '=', 'odt'),
+                ('name', '=', self.report_name),
+                ('value', '!=', ''),
+                ('value', '!=', False),
+                ('fuzzy', '=', False),
+                ('res_id', '=', 0),
+                ])
+            for translation in self.translation.browse(self.cursor, self.user,
+                    translation_ids):
+                self.cache[self.language][translation.src] = translation.value
+        return self.cache[self.language].get(text, text)
 
     def set_language(self, language):
         self.language = language
@@ -133,7 +144,7 @@ class Report(object):
         localcontext['datetime'] = datetime
         localcontext.update(context)
 
-        translate = TranslateFactory(cursor, self._name,
+        translate = TranslateFactory(cursor, user, self._name,
                 context.get('language', 'en_US'),
                 self.pool.get('ir.translation'))
         localcontext['setLang'] = lambda language: translate.set_language(language)
@@ -196,11 +207,15 @@ class Report(object):
         content_io.close()
         outzip.close()
 
-        translator = Translator(translate)
+        # Since Genshi >= 0.6, Translator requires a function type
+        translator = Translator(lambda text: translate(text))
 
         rel_report = relatorio.reporting.Report(path, 'application/vnd.oasis.opendocument.text',
                 ReportFactory(), relatorio.reporting.MIMETemplateLoader())
         rel_report.filters.insert(0, translator)
+        #convert unicode key into str
+        localcontext = dict(map(lambda x: (str(x[0]), x[1]),
+            localcontext.iteritems()))
         #Test compatibility with old relatorio version <= 0.3.0
         if len(inspect.getargspec(rel_report.__call__)[0]) == 2:
             data = rel_report(objects, **localcontext).render().getvalue()
@@ -278,7 +293,7 @@ class Report(object):
         return None
 
     def format_lang(self, cursor, user, value, lang, digits=2, grouping=True,
-            monetary=False, date=False, currency=None):
+            monetary=False, date=False, currency=None, symbol=True):
         lang_obj = self.pool.get('ir.lang')
 
         if date:
@@ -299,11 +314,13 @@ class Report(object):
                     value = str(value)[:19]
                     locale_format = locale_format + ' %H:%M:%S'
                     string_pattern = '%Y-%m-%d %H:%M:%S'
-                date = mx.DateTime.strptime(str(value), string_pattern)
+                date = datetime.datetime(*time.strptime(str(value),
+                    string_pattern)[:6])
             else:
-                date = mx.DateTime.DateTime(*(value.timetuple()[:6]))
-            return lang_obj.strftime(date.tuple(), code, locale_format)
+                date = datetime.datetime(*(value.timetuple()[:6]))
+            return lang_obj.strftime(date, code, locale_format)
         if currency:
-            return lang_obj.currency(lang, value, currency, grouping=grouping)
+            return lang_obj.currency(lang, value, currency, grouping=grouping,
+                    symbol=symbol)
         return lang_obj.format(lang, '%.' + str(digits) + 'f', value,
                 grouping=grouping, monetary=monetary)
