@@ -4,15 +4,13 @@
 """
 Miscelleanous tools used by tryton
 """
-import os, time, sys
+import os, time
 import inspect
 from trytond.config import CONFIG
-import socket
 import subprocess
 import zipfile
 from trytond.backend import Database
 from threading import Lock, local
-import logging
 import smtplib
 try:
     import cStringIO as StringIO
@@ -20,6 +18,7 @@ except ImportError:
     import StringIO
 import dis
 import datetime
+from decimal import Decimal
 
 def find_in_path(name):
     if os.name == "nt":
@@ -62,10 +61,8 @@ def exec_pg_command_pipe(name, *args):
     child_env = dict(os.environ)
     if CONFIG['db_password']:
         child_env['PGPASSWORD'] = CONFIG['db_password']
-    pipe = subprocess.Popen(cmd, shell=True,
-                            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-                            close_fds=True,
-                            env=child_env)
+    pipe = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
+            stdout=subprocess.PIPE, env=child_env)
     return pipe
 
 def exec_command_pipe(name, *args):
@@ -81,16 +78,21 @@ def exec_command_pipe(name, *args):
 def file_open(name, mode="r", subdir='modules'):
     """Open a file from the root dir, using a subdir folder."""
     from trytond.modules import EGG_MODULES
-    root_path = os.path.dirname(os.path.dirname(__file__))
+    root_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 
     name3 = False
     if subdir == 'modules':
         module_name = name.split(os.sep)[0]
         if module_name in EGG_MODULES:
-            ep = EGG_MODULES[module_name]
-            mod_path = os.path.join(ep.dist.location,
-                    *ep.module_name.split('.')[:-1])
+            epoint = EGG_MODULES[module_name]
+            mod_path = os.path.join(epoint.dist.location,
+                    *epoint.module_name.split('.')[:-1])
             name3 = os.path.join(mod_path, name)
+            if not os.path.isfile(name3):
+                # When testing modules from setuptools location is the module
+                # directory
+                name3 = os.path.join(os.path.dirname(epoint.dist.location),
+                        name)
 
     if subdir:
         if subdir == 'modules'\
@@ -98,7 +100,7 @@ def file_open(name, mode="r", subdir='modules'):
                     or name.startswith('workflow' + os.sep) \
                     or name.startswith('res' + os.sep) \
                     or name.startswith('webdav' + os.sep) \
-                    or name.startswith('tests' + os.sep)):
+                    or name.startswith('test' + os.sep)):
             name = os.path.join(root_path, name)
         else:
             name = os.path.join(root_path, subdir, name)
@@ -127,7 +129,7 @@ def file_open(name, mode="r", subdir='modules'):
                 name2 = os.path.normpath(os.path.join(head + '.zip', zipname))
     for i in (name2, name, name3):
         if i and os.path.isfile(i):
-            return file(i, mode)
+            return open(i, mode)
 
     raise IOError, 'File not found : '+str(name)
 
@@ -152,20 +154,6 @@ def get_smtp_server():
 
     return smtp_server
 
-def sms_send(user, password, api_id, text, to):
-    "text must be latin-1 encoded"
-    import urllib
-    params = urllib.urlencode({
-        'user': user,
-        'password': password,
-        'api_id': api_id,
-        'text': text,
-        'to':to,
-        })
-    #urllib.urlopen("http://api.clickatell.com/http/sendmsg", params)
-    urllib.urlopen("http://196.7.150.220/http/sendmsg", params)
-    return True
-
 def find_language_context(args, kargs=None):
     if kargs is None:
         kargs = {}
@@ -327,7 +315,8 @@ class Cache(object):
                     cursor.execute('UPDATE ir_cache SET "timestamp" = %s '\
                             'WHERE name = %s', (datetime.datetime.now(), name))
                 else:
-                    cursor.execute('INSERT INTO ir_cache ("timestamp", "name") ' \
+                    cursor.execute('INSERT INTO ir_cache ' \
+                            '("timestamp", "name") ' \
                             'VALUES (%s, %s)', (datetime.datetime.now(), name))
             Cache._resets[dbname].clear()
         finally:
@@ -335,6 +324,67 @@ class Cache(object):
             cursor.close()
             Cache._resets_lock.release()
 
+def memoize(maxsize):
+    """
+    Decorator to 'memoize' a function - caching its results with a
+    near LRU implementation.
+
+    The cache keeps a list of keys logicaly separated in 4 segment :
+
+    segment 1 | ...        | segment4
+    [k,k,k,k,k,k,k, .. ,k,k,k,k,k,k,k]
+
+    For each segment there is a pointer that loops on it.  When a key
+    is accessed from the cache it is promoted to the first segment (at
+    the pointer place of segment one), the key under the pointer is
+    moved to the next segment, the pointer is then incremented and so
+    on. A key that is removed from the last segment is removed from
+    the cache.
+
+    :param: maxsize the size of the cache (must be greater than or
+    equal to 4)
+    """
+    assert maxsize >= 4, "Memoize cannot work if maxsize is less than 4"
+
+    def wrap(fct):
+        cache = {}
+        keys = [None for i in xrange(maxsize)]
+        seg_size = maxsize // 4
+
+        pointers = [i * seg_size for i in xrange(4)]
+        max_pointers = [(i + 1) * seg_size for i in xrange(3)] + [maxsize]
+
+        def wrapper(*args):
+            key = repr(args)
+            res = cache.get(key)
+            if res:
+                pos, res = res
+                keys[pos] = None
+            else:
+                res = fct(*args)
+
+            value = res
+            for segment, pointer in enumerate(pointers):
+                newkey = keys[pointer]
+                keys[pointer] = key
+                cache[key] = (pointer, value)
+
+                pointers[segment] = pointer + 1
+                if pointers[segment] == max_pointers[segment]:
+                    pointers[segment] = segment * seg_size
+
+                if newkey is None:
+                    break
+                segment, value = cache.pop(newkey)
+                key = newkey
+
+            return res
+
+        wrapper.__doc__ = fct.__doc__
+        wrapper.__name__ = fct.__name__
+
+        return wrapper
+    return wrap
 
 def mod10r(number):
     """
@@ -355,103 +405,102 @@ def mod10r(number):
 
 class LocalDict(local):
 
-    def __init__(self, dict=None):
-        if dict is None:
-            dict = {}
-        self.dict = dict
+    def __init__(self):
+        super(LocalDict, self).__init__()
+        self._dict = {}
 
     def __str__(self):
-        return str(self.dict)
+        return str(self._dict)
 
     def __repr__(self):
-        return str(self.dict)
+        return str(self._dict)
 
     def clear(self):
-        return self.dict.clear()
+        return self._dict.clear()
 
     def keys(self):
-        return self.dict.keys()
+        return self._dict.keys()
 
     def __setitem__(self, i, y):
-        self.dict.__setitem__(i, y)
+        self._dict.__setitem__(i, y)
 
     def __getitem__(self, i):
-        return self.dict.__getitem__(i)
+        return self._dict.__getitem__(i)
 
     def copy(self):
-        return self.dict.copy()
+        return self._dict.copy()
 
     def iteritems(self):
-        return self.dict.iteritems()
+        return self._dict.iteritems()
 
     def iterkeys(self):
-        return self.dict.iterkeys()
+        return self._dict.iterkeys()
 
     def itervalues(self):
-        return self.dict.itervalues()
+        return self._dict.itervalues()
 
     def pop(self, k, d=None):
-        return self.dict.pop(k, d)
+        return self._dict.pop(k, d)
 
     def popitem(self):
-        return self.dict.popitem()
+        return self._dict.popitem()
 
     def setdefault(self, k, d=None):
-        return self.dict.setdefault(k, d)
+        return self._dict.setdefault(k, d)
 
     def update(self, E, **F):
-        return self.dict.update(E, F)
+        return self._dict.update(E, F)
 
     def values(self):
-        return self.dict.values()
+        return self._dict.values()
 
     def get(self, k, d=None):
-        return self.dict.get(k, d)
+        return self._dict.get(k, d)
 
     def has_key(self, k):
-        return self.dict.has_key(k)
+        return self._dict.has_key(k)
 
     def items(self):
-        return self.dict.items()
+        return self._dict.items()
 
     def __cmp__(self, y):
-        return self.dict.__cmp__(y)
+        return self._dict.__cmp__(y)
 
     def __contains__(self, k):
-        return self.dict.__contains__(k)
+        return self._dict.__contains__(k)
 
     def __delitem__(self, y):
-        return self.dict.__delitem__(y)
+        return self._dict.__delitem__(y)
 
     def __eq__(self, y):
-        return self.dict.__eq__(y)
+        return self._dict.__eq__(y)
 
     def __ge__(self, y):
-        return self.dict.__ge__(y)
+        return self._dict.__ge__(y)
 
     def __getitem__(self, y):
-        return self.dict.__getitem__(y)
+        return self._dict.__getitem__(y)
 
     def __gt__(self, y):
-        return self.dict.__gt__(y)
+        return self._dict.__gt__(y)
 
     def __hash__(self):
-        return self.dict.__hash__()
+        return self._dict.__hash__()
 
     def __iter__(self):
-        return self.dict.__iter__()
+        return self._dict.__iter__()
 
     def __le__(self, y):
-        return self.dict.__le__(y)
+        return self._dict.__le__(y)
 
     def __len__(self):
-        return self.dict.__len__()
+        return self._dict.__len__()
 
     def __lt__(self, y):
-        return self.dict.__lt__(y)
+        return self._dict.__lt__(y)
 
     def __ne__(self, y):
-        return self.dict.__ne__(y)
+        return self._dict.__ne__(y)
 
 def reduce_ids(field, ids):
     '''
@@ -461,6 +510,8 @@ def reduce_ids(field, ids):
     :param ids: the list of ids
     :return: sql string and sql param
     '''
+    if not ids:
+        return '(%s)', [False]
     ids = ids[:]
     ids.sort()
     prev = ids.pop(0)
@@ -506,19 +557,18 @@ _ALLOWED_CODES = set(dis.opmap[x] for x in [
     'BINARY_LSHIFT','BINARY_RSHIFT','BINARY_AND','BINARY_XOR', 'BINARY_OR',
     'STORE_MAP', 'LOAD_NAME', 'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR',
     'STORE_NAME', 'GET_ITER', 'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE',
-    'DELETE_NAME', 'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'BINARY_SUBSCR',
+    'DELETE_NAME', 'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
+    'BINARY_SUBSCR',
     ] if x in dis.opmap)
 
-
-def safe_eval(source, data=None):
-    if '__subclasses__' in source:
-        raise ValueError('__subclasses__ not allowed')
-    c = compile(source, '', 'eval')
+@memoize(1000)
+def _compile_source(source):
+    comp = compile(source, '', 'eval')
     codes = []
-    s = c.co_code
+    co_code = comp.co_code
     i = 0
-    while i < len(s):
-        code = ord(s[i])
+    while i < len(co_code):
+        code = ord(co_code[i])
         codes.append(code)
         if code >= dis.HAVE_ARGUMENT:
             i += 3
@@ -527,7 +577,14 @@ def safe_eval(source, data=None):
     for code in codes:
         if code not in _ALLOWED_CODES:
             raise ValueError('opcode %s not allowed' % dis.opname[code])
-    return eval(c, {'__builtins__': {
+    return comp
+
+def safe_eval(source, data=None):
+    if '__subclasses__' in source:
+        raise ValueError('__subclasses__ not allowed')
+
+    comp = _compile_source(source)
+    return eval(comp, {'__builtins__': {
         'True': True,
         'False': False,
         'str': str,
@@ -535,4 +592,6 @@ def safe_eval(source, data=None):
         'locals': locals,
         'bool': bool,
         'dict': dict,
+        'round': round,
+        'Decimal': Decimal,
         }}, data)
